{-*-----------------------------------------------------------------------
  The Core Assembler.

  Daan Leijen.

  Copyright 2001, Daan Leijen. All rights reserved. This file
  is distributed under the terms of the GHC license. For more
  information, see the file "license.txt", which is included in
  the distribution.
-----------------------------------------------------------------------*-}

-- $Id: Special.hs,v 1.4 2002/07/22 13:40:39 cvs-3 Exp $

{---------------------------------------------------------------
  Special definitions for the GHC system
---------------------------------------------------------------}
module Special( doesFileExist
              , openBinary, closeBinary, readBinary, writeBinaryChar
              , ST, STArray, runST, newSTArray, readSTArray, writeSTArray
              , unsafeCoerce
              ) where

import Directory( doesFileExist )
import IO       ( Handle, hGetContents, hClose, hPutChar, IOMode(..) )
import IOExts   ( openFileEx, IOModeEx(..))

#if (__GLASGOW_HASKELL__ >= 503)
import GHC.Base ( unsafeCoerce# )
import ST       ( ST, STArray, runST, newSTArray, readSTArray, writeSTArray)
#else
import GlaExts  ( unsafeCoerce# )
import LazyST   ( ST, STArray, runST, newSTArray, readSTArray, writeSTArray)
#endif
              
unsafeCoerce :: a -> b
unsafeCoerce x
  = unsafeCoerce# x

openBinary :: FilePath -> IOMode -> IO Handle
openBinary path mode
  = openFileEx path (BinaryMode mode)

closeBinary :: Handle -> IO ()
closeBinary h
  = hClose h

readBinary :: Handle -> IO String
readBinary h
  = do{ xs <- hGetContents h
      ; seq (last xs) $ hClose h
      ; return xs
      }

writeBinaryChar :: Handle -> Char -> IO ()
writeBinaryChar h c
  = hPutChar h c
